import os
import sys
import time
import subprocess
file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)

command = ["cat", "/proc/cpuinfo"]
output = subprocess.check_output(command)
for line in output.decode().splitlines():
    if "Model" in line:
        model = line.split(":")[1].strip()
        break    
#print(model)
if model.find("Raspberry Pi 5") != -1:
    from BASE5 import * 
else:
    from BASE0 import *

tempScale='f'

tempt=time.time()
tempTime = [[tempt for x in range(8)] for y in range(8)]
tempLast = [[0 for x in range(8)] for y in range(8)]
TEMPTHROTTLE=1.0

def getTEMP(addr,chan,scale=None):
    global tempScale
    VerifyADDR(addr)	
    assert ((chan>=1) and (chan<=8)),"channel number out of range. Must be between 1 and 8"    
    chan = chan-1
    if scale is None:
        scal=tempScale
    else:
        scal=scale.lower()
        assert ((scal=='c') or (scal=='f') or (scal=='k')), "Temperature scale must be 'c', 'f', or 'k'."
    if ((time.time()-tempTime[addr][chan]) >= TEMPTHROTTLE):
        tempTime[addr][chan]=time.time()
        resp=ppCMD(addr,0x71,chan,0,2)

    #resp=ppCMD(addr,0x71,chan-1,0,2)   #get data
    
        Temp=resp[0]*256+resp[1]
        if (Temp>0x8000):
            Temp = Temp^0xFFFF
            Temp = -(Temp+1)
        Temp = round((Temp/16.0),4)
        tempLast[addr][chan]=Temp       #save last temp read in Celcius units
    Temp=tempLast[addr][chan]
    if (scal=='k'):
        Temp = Temp + 273
    if (scal=='f'):
        Temp = round((Temp*1.8+32.2),4)
    #return Temp
    return Temp
    
def setSCALE(scale):
    global tempScale
    scal=scale.lower()
    assert ((scal=='c') or (scal=='f') or (scal=='k')), "Temperature scale must be 'c', 'f', or 'k'."
    tempScale=scal
    
def getSCALE():
    global tempScale
    return tempScale


import contextlib
with contextlib.redirect_stdout(None):
    import pygame
pygame.init()

MAXWIDTH=1000
MAXHEIGHT=100
MAXLINES=12
FONTSIZE=128
COLOR = (0,128,0)
units=['units']*MAXLINES
val=[0.0]*MAXLINES
desc=['']*MAXLINES
lines=1
size = width, height = MAXWIDTH, MAXHEIGHT
meterOpen=False

def openMETER(linesel=None):
    global font, screen, size, height, width, lines, units, val, meterOpen
    if linesel is None:
        size = width, height = MAXWIDTH, MAXHEIGHT
        lines=1
    else:    
        assert ((linesel>=1) and (linesel<=MAXLINES)), "Line count must be between 1 and "+str(MAXLINES)
        size = width, height = MAXWIDTH, MAXHEIGHT*linesel
        lines=linesel
    screen = pygame.display.set_mode(size)
    pygame.display.set_caption('TINKERplate Meter')
    font=pygame.font.SysFont(None,FONTSIZE)  
    refresh()
    meterOpen=True
    
def closeMETER():
    global meterOpen
    if(meterOpen):
        pygame.display.quit()
        meterOpen=False
         
def setMETER(value,unit,descriptor,linesel=None):
    global size, height, width, lines, units, val, desc
    assert(meterOpen), "You must open a meter before you write to it."
    if linesel is None:
        line=1
    else: 
        assert (linesel<=lines),"That line does not exist. Close your current meter and reopen with the correct number of lines."
        assert ((linesel>=1) and (linesel<=MAXLINES)), "Line count must be between 1 and "+str(MAXLINES)
        line=linesel  
    desc[line-1]=descriptor
    val[line-1]=value
    units[line-1]=unit
    refresh()

def setTITLE(title):
    pygame.display.set_caption(title)
    
def setCOLOR(newcolor):
    global COLOR
    assert (len(newcolor)==3), "Only three arguments accepted for color."
    assert (newcolor[0]>=0 and newcolor[0]<=255), "Argument must be between 0 and 255"
    assert (newcolor[1]>=0 and newcolor[1]<=255), "Argument must be between 0 and 255"
    assert (newcolor[2]>=0 and newcolor[2]<=255), "Argument must be between 0 and 255"    
    COLOR=newcolor
    
def refresh():
    global acreen, size, height, width, lines, units, val, desc
    screen.fill((0,0,0))
    for i in range(lines):
        text_surface = font.render(desc[i]+str("%7.3f"%(val[i]))+' '+units[i], True, COLOR)
        text_rect = text_surface.get_rect(center=(width/2, height*(2*i+1)/(2*lines)))       
        screen.blit(text_surface,text_rect)
    for i in range(lines):        
        pygame.draw.line(screen,COLOR,[0,(i+1)*height/lines],[width,(i+1)*height/lines],3)           
    pygame.display.update()


import spidev
import time
#import string
import sys
import os
import subprocess
import gpiod


FRAME = 25
ACK = 23
SRQ = 22
SW = 24
drMAP=[5,6,13,19,26,21,20,16]
dedSRQ=8*[0]


chip=gpiod.Chip('gpiochip4')

ppFRAME=chip.get_line(FRAME)
ppSRQ=chip.get_line(SRQ)
ppACK=chip.get_line(ACK)
ppSW=chip.get_line(SW)
ppFRAME.request(consumer="Frame", type=gpiod.LINE_REQ_DIR_OUT)
ppSRQ.request(consumer="SRQ", type=gpiod.LINE_REQ_DIR_IN, flags=gpiod.LINE_REQ_FLAG_BIAS_PULL_UP)
ppACK.request(consumer="ACK", type=gpiod.LINE_REQ_DIR_IN, flags=gpiod.LINE_REQ_FLAG_BIAS_PULL_UP)
ppSW.request(consumer="SW", type=gpiod.LINE_REQ_DIR_IN, flags=gpiod.LINE_REQ_FLAG_BIAS_PULL_UP)

ppFRAME.set_value(0)

try:
    spi = spidev.SpiDev()
    spi.open(0,1)
except:
    print("Did you enable the SPI hardware interface on your Raspberry Pi?")
    print("Go to https://pi-plates.com/getting_started/ and learn how.")

DataGood=True

def enDedicated(addr):
    dedSRQ[addr]=chip.get_line(drMAP[addr])
    dedSRQ[addr].request(consumer="dSRQ"+str(addr), type=gpiod.LINE_REQ_DIR_IN, flags=gpiod.LINE_REQ_FLAG_BIAS_PULL_UP)

def disDedicated(addr):
    try:
        dedSRQ[addr].release()
        dedSRQ[addr]=0
    except:
        pass

def getDedicated(addr):
    try:
        stat=False
        if (dedSRQ[addr].get_value()==0):
            stat=True
        return stat
    except:
        return False
    
def getSRQ():
    if (ppSRQ.get_value()==1):
        val=False
    else:
        val=True
    #print(ppSRQ.get_value(), val)
    return val

def CLOSE():
    spi.close()
    ppFRAME.release()
    ppACK.release()
    ppSRQ.release()

def ppCMD1(addr,cmd,param1,param2,bytes2return):
    #global GPIObaseADDR
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    #    time.sleep(.0005)
    ppFRAME.set_value(1)
    null=spi.xfer(arg,300000,40)
    #null = spi.writebytes(arg)   
    if bytes2return>0:
        time.sleep(.0001)        
        for i in range(0,bytes2return):	
            dummy=spi.xfer([00],500000,20)
            resp.append(dummy[0])        
    ppFRAME.set_value(0)
    time.sleep(.0003)
    return resp	

def ppCMD2(addr,cmd,param1,param2,bytes2return):
    global DataGood
    DataGood=True
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    ppFRAME.set_value(1)       #Set FRAME high - tell Pi-Plates to start listening
    null=spi.xfer(arg,500000,5)     #Send out 4 byte command - ignore what comes back 
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (ppACK.get_value()!=1):  #wait 50msec for the addressed DAQC2 to ACKnowledge command
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False    
    if (bytes2return>0) and DataGood:   #If DAQC2 is supposed to send data AND no timeout occurred
        t0=time.time()
        wait=True
        while(wait):
            if (ppACK.get_value()!=1):  #Ensure that ACK is still low before collecting data             
                wait=False
            if ((time.time()-t0)>0.08): #timeout
                wait=False
                DataGood=False
        if (DataGood==True):            #if ACK is still low AND there was no timeout then fetch data
            #time.sleep(.0001)
            for i in range(0,bytes2return+1):	#Fetch each byte. That [00] is simply a single element list set to zero
                dummy=spi.xfer([00],500000,5)   #That [00] is simply a single element list set to zero
                resp.append(dummy[0])           
            csum=0;                             
            for i in range(0,bytes2return):     #calculate and verify checksum
                csum+=resp[i]
            if ((~resp[bytes2return]& 0xFF) != (csum & 0xFF)):
                DataGood=False
    ppFRAME.set_value(0)
    time.sleep(.00001)       #added for the RPi5 which is damn fast. DAQC2plate would sometimes miss FRAME going from high to low
    return resp

    
def getID1(addr):
    id=""
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;
    ppFRAME.set_value(1)
    #null = spi.writebytes(arg)
    null=spi.xfer(arg,300000,60)
    count=0
#    time.sleep(.0001)
    while (count<20): 
        dummy=spi.xfer([00],300000,20)
#        time.sleep(.0001)
        if (dummy[0] != 0):
            num = dummy[0]
            id = id + chr(num)
            count = count + 1
        else:
            count=20
    ppFRAME.set_value(0)
    return id

def getID2(addr):
    id=""
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;
    ppFRAME.set_value(1)
    null=spi.xfer(arg,500000,50)
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (ppACK.get_value()!=1):              
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    if (DataGood==True):
        count=0 
        csum=0
        go=True
        while (go): 
            dummy=spi.xfer([00],500000,40)
            if (dummy[0] != 0):
                num = dummy[0]
                csum += num
                id = id + chr(num)
                #print count, num
                count += 1
            else:
                dummy=spi.xfer([00],500000,40)  
                checkSum=dummy[0]                
                go=False 
            if (count>25):
                go=False
                DataGood=False
        if ((~checkSum & 0xFF) != (csum & 0xFF)):
            DataGood=False
    ppFRAME.set_value(0)
    time.sleep(.00001)       #added for the RPi5 which is damn fast. DAQC2plate would sometimes miss FRAME going from high to low
    return id       

def ppCMDADC(addr,cmd,param1,param2,bytes2return,slow=None):
    global DataGood
    if (slow==None):
        tOut=0.05
    else:
        tOut=3.0
    arg = list(range(4))
    resp = [0]*(bytes2return+1)
    arg[0]=addr;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    DataGood=True
    wait=True
    t0=time.time()
    while(wait):
        if (ppACK.get_value()!=0):
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    ppFRAME.set_value(1)
    time.sleep(0.000001)     #allow the uP some time to initialize the SPI
    spi.xfer(arg,400000,0)
    t0=time.time()
    wait=True
    while(wait):
        if (ppACK.get_value()!=1):
            wait=False
        if ((time.time()-t0)>tOut):   #timeout
            wait=False
            DataGood=False
    if (bytes2return>0) and DataGood:
        time.sleep(0.000001)     #allow the uP some time to initialize the SPI
        t0=time.time()
        wait=True
        while(wait):
            if (ppACK.get_value()!=1):
                wait=False
            if ((time.time()-t0)>0.1):   #timeout
                wait=False
                DataGood=False
        if (DataGood==True):
            resp=spi.xfer([0]*(bytes2return+1),4000000,0)   #don't exceed 4Mhz
            csum=0;
            for i in range(0,bytes2return+1):
                csum+=resp[i]
            if ((csum & 0xFF) != 0xFF):
                DataGood=False
    ppFRAME.set_value(0)
    time.sleep(0.000001)     #allow the uP some time to close SPI engine
    return resp

def ppCMDosc(addr,cmd,param1,param2,bytes2return):
    global DataGood
    DataGood=True
    arg = list(range(4))
    resp = [0]*(bytes2return)
    N=bytes2return>>6
    arg[0]=addr;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    ppFRAME.set_value(1)            #Set FRAME high - tell Pi-Plates to start listening
    null=spi.xfer(arg,500000,5)     #Send out 4 byte command - ignore what comes back 
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (ppACK.get_value()!=1):         #wait 50msec for the addressed DAQC2 to ACKnowledge command
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False    
    if (bytes2return>0) and DataGood:   #If DAQC2 is supposed to send data AND no timeout occurred
        t0=time.time()
        wait=True
        while(wait):
            if (ppACK.get_value()!=1):  #Ensure that ACK is still low before collecting data             
                wait=False
            if ((time.time()-t0)>0.08): #timeout
                wait=False
                DataGood=False
        if (DataGood==True):            #if ACK is still low AND there was no timeout then fetch data
            for i in range(N):          # bug in RPi5 SPIDEV limits block transfers to 64 bytes            
                resp[i*64:i*64+63]=spi.xfer([0]*(64),2000000) 
                #print(len(resp))
    ppFRAME.set_value(0)
    time.sleep(.00001)                  #added for the RPi5 which is damn fast. DAQC2plate would sometimes miss FRAME going from high to low
    return resp
    
def fetchBLOCK(addr,cmd,param1,param2,bytes2return):
    global DataGood
    blockBYTES=[0]*bytes2return
    REM=bytes2return&0x3F
    N=bytes2return>>6
    
    arg = list(range(4))
    arg[0]=addr
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    DataGood=True
    wait=True
    t0=time.time()
    while(wait):
        if (ppACK.get_value()!=0):  #Ensure that ACK is high before proceeding
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    ppFRAME.set_value(1)       #Set the FRAME signal high
    time.sleep(0.000001)       #allow the uP some time to initialize the SPI
    spi.xfer(arg,500000,0)     #Send the fetch command
    t0=time.time()
    wait=True
    while(wait):
        if (ppACK.get_value()==0):  #wait until the command is received and ACKnowledged
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    if DataGood:
        time.sleep(0.000001)     #allow the uP some time to initialize the SPI
        if(N>0):
            for i in range(N):          # bug in RPi5 SPIDEV limits block transfers to 64 bytes            
                blockBYTES[i*64:i*64+63]=spi.xfer3([0x55]*64,4000000,0)
            if (REM != 0):
                blockBYTES[N*64:N*64+(REM-1)]=spi.xfer3([0x55]*REM,4000000,0)
        else:
            if (REM != 0):
                blockBYTES[0:(REM-1)]=spi.xfer3([0x55]*REM,4000000,0)
    ppFRAME.set_value(0)
    time.sleep(.00001)          #added for the RPi5 which is damn fast. DAQC2plate would sometimes miss FRAME going from high to low    
    return list(blockBYTES)


import os
import sys
import subprocess
file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)

command = ["cat", "/proc/cpuinfo"]
output = subprocess.check_output(command)
for line in output.decode().splitlines():
    if "Model" in line:
        model = line.split(":")[1].strip()
        break    
#print(model)
if model.find("Raspberry Pi 5") != -1:
    from BASE5 import * 
else:
    from BASE0 import *

#==============================================================================#	
# RELAY Functions	                                                           #
#==============================================================================#  
def relayON(addr,relay):
    VerifyADDR(addr)
    assert ((relay>=1) and (relay<=2)),"Relay number out of range. Must be between 1 and 2"
    relay -= 1
    ppCMD(addr,0x10,relay,0,0)

def relayOFF(addr,relay):
    VerifyADDR(addr)
    assert ((relay>=1) and (relay<=2)),"Relay number out of range. Must be between 1 and 2"
    relay -= 1
    ppCMD(addr,0x11,relay,0,0)
    
def relayTOGGLE(addr,relay):
    VerifyADDR(addr)
    assert ((relay>=1) and (relay<=2)),"Relay number out of range. Must be between 1 and 2"
    relay -= 1
    ppCMD(addr,0x12,relay,0,0)   

def relayALL(addr,relays):
    VerifyADDR(addr)
    assert ((relays>=0) and (relays<=3)),"Argument out of range. Must be between 0 and 3"
    ppCMD(addr,0x13,relays,0,0) 
    
def relaySTATE(addr,relay):
    VerifyADDR(addr)
    assert ((relay>=1) and (relay<=2)),"Relay number out of range. Must be between 1 and 2"
    relay -= 1
    resp=ppCMD(addr,0x14,relay,0,1) 
    return resp[0]
    



import os
import sys
import subprocess

file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)

command = ["cat", "/proc/cpuinfo"]
output = subprocess.check_output(command)
for line in output.decode().splitlines():
    if "Model" in line:
        model = line.split(":")[1].strip()
        break    
#print(model)
if model.find("Raspberry Pi 5") != -1:
    from BASE5 import * 
else:
    from BASE0 import *

#==============================================================================#	
# SERVO Functions	                                                           #
#==============================================================================# 

SERVOlow=0.6
SERVOhigh=2.35

def setSERVO(addr,servo,angle):
    global SERVOlow
    global SERVOhigh
    VerifyADDR(addr)
    assert ((angle>=0) and (angle<=180)),"Angle value out of range. Must be between 0 and 180"
    servo -= 1
    #clockDec=1e-3*(1.0+angle/180.0)
    clockDec=1e-3*(SERVOlow+(SERVOhigh-SERVOlow)*(180-angle)/180.0)
    clockDec=int(clockDec*49e6/12.0)
    clockStart=2**16-clockDec
    hbyte=clockStart>>8
    lbyte=clockStart & 0xFF
    #print clockStart, hbyte, lbyte 
    ppCMD(addr,0x50+servo,hbyte,lbyte,0)
    
def setSERVO2(addr,servo,pw):
    VerifyADDR(addr)
    assert ((servo>=1) and (servo<=8)),"Servo number out of range. Must be between 1 and 8"
    if (pw>2.45):
        pw=2.45
    if (pw<0.5):
        pw=0.5
    servo -= 1
    clockDec=pw*1e-3
    clockDec=int(clockDec*49e6/12.0)
    clockStart=2**16-clockDec
    hbyte=clockStart>>8
    lbyte=clockStart & 0xFF
    #print clockStart, hbyte, lbyte 
    ppCMD(addr,0x50+servo,hbyte,lbyte,0)

def setSERVOlow(addr,value):
    global SERVOlow
    if (value>2.45):
        value=2.45
    if (value<0.5):
        value=0.5
    SERVOlow=value
    
def setSERVOhigh(addr,value):
    global SERVOhigh
    if (value>2.45):
        value=2.45
    if (value<0.5):
        value=0.5
    SERVOhigh=value    
    


#==============================================================================#	
# BASIC I/O Functions	                                                       #
#==============================================================================#          
import time
import spidev
import gpiod
   
FRAME = 25
ACK = 23
SRQ = 22
       
ppFRAME=chip.get_line(FRAME)
ppSRQ=chip.get_line(SRQ)
ppACK=chip.get_line(ACK)
ppFRAME.request(consumer="Frame", type=gpiod.LINE_REQ_DIR_OUT)
ppSRQ.request(consumer="SRQ", type=gpiod.LINE_REQ_DIR_IN, flags=gpiod.LINE_REQ_FLAG_BIAS_PULL_UP)
ppACK.request(consumer="ACK", type=gpiod.LINE_REQ_DIR_IN, flags=gpiod.LINE_REQ_FLAG_BIAS_PULL_UP)
#ppACK.request(consumer="ACK", type=gpiod.LINE_REQ_DIR_IN)
ppFRAME.set_value(0)

spi = spidev.SpiDev()
spi.open(0,1)
baseADDR=48

baseADDR=48
ppNAME='TINKERplate'
MAXADDR=8

DataGood=False
platesPresent = list(range(8))

def VerifyADDR(addr):
    assert ((addr>=0) and (addr<MAXADDR)),ppNAME+" address out of range"
    addr_str=str(addr)
    assert (platesPresent[addr]==1),"No "+ppNAME+" found at address "+addr_str
    
def VerifyCHAN(chan):
    assert ((chan>=1) and (chan<9)),"Invalid channel number - must be between 1 and 8"
  
def ppCMD(addr,cmd,param1,param2,bytes2return):
    global baseADDR
    global DataGood
    DataGood=True
    arg = list(range(4))
    resp = []
    arg[0]=addr+baseADDR;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    DataGood=True
    # t0=time.time()
    # wait=True    
    # while(wait):
        # if (ppACK.get_value()!=1):              
            # wait=False
        # if ((time.time()-t0)>0.05):   #timeout
            # wait=False
            # DataGood=False
    # if (DataGood==True): 
    ppFRAME.set_value(1)       #Set FRAME high - tell Pi-Plates to start listening
    null=spi.xfer(arg,500000,5)
    #DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (ppACK.get_value()!=1):
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False    
    if (bytes2return>0) and DataGood:
        t0=time.time()
        wait=True
        while(wait):
            if (ppACK.get_value()!=1):              
                wait=False
            if ((time.time()-t0)>0.08):   #timeout
                wait=False
                DataGood=False
        if (DataGood==True):
            for i in range(0,bytes2return+1):	
                dummy=spi.xfer([00],500000,5)
                resp.append(dummy[0])
            csum=0;
            for i in range(0,bytes2return):
                csum+=resp[i]
            if ((~resp[bytes2return]& 0xFF) != (csum & 0xFF)):
                DataGood=False
    ppFRAME.set_value(0)
    time.sleep(.00001)       #added for the RPi5 which is damn fast. DAQC2plate would sometimes miss FRAME going from high to low
    return resp

def getID(addr):
    global baseADDR
    VerifyADDR(addr)
    addr=addr+baseADDR
    id=""
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;
    ppFRAME.set_value(1)
    null=spi.xfer(arg,500000,50)
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (ppACK.get_value()!=1):              
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    if (DataGood==True): 
        count=0 
        csum=0
        go=True
        while (go): 
            dummy=spi.xfer([00],500000,40)
            if (dummy[0] != 0):
                num = dummy[0]
                csum += num
                id = id + chr(num)
                count += 1
            else:
                dummy=spi.xfer([00],500000,40)  
                checkSum=dummy[0]                
                go=False 
            if (count>25):
                go=False
                DataGood=False
        #print checkSum, ~checkSum & 0xFF, csum & 0xFF
        if ((~checkSum & 0xFF) != (csum & 0xFF)):
            DataGood=False
    ppFRAME.set_value(0)
    time.sleep(.00001)       #added for the RPi5 which is damn fast. DAQC2plate would sometimes miss FRAME going from high to low
    return id   
    
def getADDR(addr):
    global baseADDR
    resp=ppCMD(addr,0x00,0,0,1)
    #print resp, DataGood;
    if (DataGood):
        return resp[0]-baseADDR
    else:
        return 8
    
def quietPoll():   
    global platesPresent
    ppFoundCount=0
    for i in range (0,8):
        platesPresent[i]=0
        rtn = getADDR(i)
        print(rtn)
        if (rtn==i):           
            platesPresent[i]=1
            ppFoundCount += 1
        
def RESET(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x0F,0,0,0)
    time.sleep(.10)


import spidev
import time
import sys
import RPi.GPIO as GPIO
import subprocess
import os

#Initialize
if (sys.version_info < (3,0,0)):
    sys.stderr.write("You need python 3 to use this library")
    exit(1)

GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)
ADCbaseADDR=64
ppFRAME = 25
ppINT = 22
ppACK = 23
ppSW = 24

GPIO.setup(ppFRAME,GPIO.OUT)
GPIO.output(ppFRAME,False)  #Initialize FRAME signal
time.sleep(.001)            #let Pi-Plate reset SPI engine if necessary
GPIO.setup(ppINT, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(ppACK, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(ppSW, GPIO.IN, pull_up_down=GPIO.PUD_UP)

drMAP=[5,6,13,19,26,21,20,16]
dedSRQ=8*[0]

try:
    spi = spidev.SpiDev()
    spi.open(0,1)
except:
    print("Did you enable the SPI hardware interface on your Raspberry Pi?")
    print("Go to https://pi-plates.com/getting_started/ and learn how.")
    
def enDedicated(addr):
    GPIO.setup(drMAP[addr], GPIO.IN, pull_up_down=GPIO.PUD_UP)

def disDedicated(addr):
    try:
        GPIO.cleanup(drMAP[addr])
        #dedSRQ[addr].release()
        dedSRQ[addr]=0
    except:
        pass

def getDedicated(addr):
    try:
        stat=False
        if (GPIO.input(drMAP[addr])==0):
            stat=True
        return stat
    except:
        return False
    
def getSRQ():
    if (GPIO.input(ppINT)==1):
        val=False
    else:
        val=True
    return val

def CLOSE():
    spi.close()
    GPIO.cleanup()
    
def ppCMD1(addr,cmd,param1,param2,bytes2return):
    #global GPIObaseADDR
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    #    time.sleep(.0005)
    GPIO.output(ppFRAME,True)
    null=spi.xfer(arg,300000,40)
    #null = spi.writebytes(arg)   
    if bytes2return>0:
        time.sleep(.0001)        
        for i in range(0,bytes2return):	
            dummy=spi.xfer([00],500000,20)
            resp.append(dummy[0])        
    GPIO.output(ppFRAME,False)
    time.sleep(.0003)
    return resp	    
  
def ppCMD2(addr,cmd,param1,param2,bytes2return):
    global DataGood
    DataGood=True
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    GPIO.output(ppFRAME,True)       #Set FRAME high - tell Pi-Plates to start listening
    null=spi.xfer(arg,500000,5)     #Send out 4 byte command - ignore what comes back 
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)!=1):  #wait 50msec for the addressed DAQC2 to ACKnowledge command
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False    
    if (bytes2return>0) and DataGood:   #If DAQC2 is supposed to send data AND no timeout occurred
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):  #Ensure that ACK is still low before collecting data             
                wait=False
            if ((time.time()-t0)>0.08): #timeout
                wait=False
                DataGood=False
        if (DataGood==True):            #if ACK is still low AND there was no timeout then fetch data
            #time.sleep(.0001)
            for i in range(0,bytes2return+1):	#Fetch each byte. That [00] is simply a single element list set to zero
                dummy=spi.xfer([00],500000,5)   #That [00] is simply a single element list set to zero
                resp.append(dummy[0])           
            csum=0;                             
            for i in range(0,bytes2return):     #calculate and verify checksum
                csum+=resp[i]
            if ((~resp[bytes2return]& 0xFF) != (csum & 0xFF)):
                DataGood=False
    GPIO.output(ppFRAME,False)
    time.sleep(.00001)       #added for the RPi5 which is damn fast. DAQC2plate would sometimes miss FRAME going from high to low
    return resp
    
def getID1(addr):
    id=""
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;
    GPIO.output(ppFRAME,True)
    #null = spi.writebytes(arg)
    null=spi.xfer(arg,300000,60)
    count=0
#    time.sleep(.0001)
    while (count<20): 
        dummy=spi.xfer([00],300000,20)
#        time.sleep(.0001)
        if (dummy[0] != 0):
            num = dummy[0]
            id = id + chr(num)
            count = count + 1
        else:
            count=20
    GPIO.output(ppFRAME,False)
    return id

def getID2(addr):
    id=""
    arg = list(range(4))
    resp = []
    arg[0]=addr;
    arg[1]=0x1;
    arg[2]=0;
    arg[3]=0;
    GPIO.output(ppFRAME,True)
    null=spi.xfer(arg,500000,50)
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)!=1):              
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    if (DataGood==True):
        count=0 
        csum=0
        go=True
        while (go): 
            dummy=spi.xfer([00],500000,40)
            if (dummy[0] != 0):
                num = dummy[0]
                csum += num
                id = id + chr(num)
                #print count, num
                count += 1
            else:
                dummy=spi.xfer([00],500000,40)  
                checkSum=dummy[0]                
                go=False 
            if (count>25):
                go=False
                DataGood=False
        if ((~checkSum & 0xFF) != (csum & 0xFF)):
            DataGood=False
    GPIO.output(ppFRAME,False)
    time.sleep(.00001)       #added for the RPi5 which is damn fast. DAQC2plate would sometimes miss FRAME going from high to low
    return id
    
def ppCMDADC(addr,cmd,param1,param2,bytes2return,slow=None):
    global DataGood
    if (slow==None):
        tOut=0.05
    else:
        tOut=3.0
    arg = list(range(4))
    resp = [0]*(bytes2return+1)
    arg[0]=addr;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    DataGood=True
    wait=True
    t0=time.time()
    while(wait):
        if (GPIO.input(ppACK)!=0):
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    GPIO.output(ppFRAME,True)
    time.sleep(0.000001)     #allow the uP some time to initialize the SPI
    spi.xfer(arg,400000,0)
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)!=1):
            wait=False
        if ((time.time()-t0)>tOut):   #timeout
            wait=False
            DataGood=False
    if (bytes2return>0) and DataGood:
        time.sleep(0.000001)     #allow the uP some time to initialize the SPI
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):
                wait=False
            if ((time.time()-t0)>0.1):   #timeout
                wait=False
                DataGood=False
        if (DataGood==True):
            resp=spi.xfer([0]*(bytes2return+1),4000000,0)   #don't exceed 4Mhz
            csum=0;
            for i in range(0,bytes2return+1):
                csum+=resp[i]
            if ((csum & 0xFF) != 0xFF):
                DataGood=False
    GPIO.output(ppFRAME,False)
    time.sleep(0.000001)     #allow the uP some time to close SPI engine
    return resp
    
def ppCMDosc(addr,cmd,param1,param2,bytes2return):
    global DataGood
    DataGood=True
    arg = list(range(4))
    resp = [0]*(bytes2return)
    N=bytes2return>>6
    print(bytes2return,N)
    arg[0]=addr;
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    GPIO.output(ppFRAME,True)            #Set FRAME high - tell Pi-Plates to start listening
    null=spi.xfer(arg,500000,5)     #Send out 4 byte command - ignore what comes back 
    DataGood=True
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)!=1):         #wait 50msec for the addressed DAQC2 to ACKnowledge command
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False    
    if (bytes2return>0) and DataGood:   #If DAQC2 is supposed to send data AND no timeout occurred
        t0=time.time()
        wait=True
        while(wait):
            if (GPIO.input(ppACK)!=1):  #Ensure that ACK is still low before collecting data             
                wait=False
            if ((time.time()-t0)>0.08): #timeout
                wait=False
                DataGood=False
        if (DataGood==True):            #if ACK is still low AND there was no timeout then fetch data
            for i in range(N):          # bug in RPi5 SPIDEV limits block transfers to 64 bytes            
                resp[i*64:i*64+63]=spi.xfer([0]*(64),2000000) 
                #print(len(resp))
    GPIO.output(ppFRAME,False)
    time.sleep(.00001)                  #added for the RPi5 which is damn fast. DAQC2plate would sometimes miss FRAME going from high to low
    return resp
    
def fetchBLOCK(addr,cmd,param1,param2,bytes2return):
    global DataGood
    blockBYTES=[0]*bytes2return
    REM=bytes2return&0x3F
    N=bytes2return>>6
    
    arg = list(range(4))
    arg[0]=addr
    arg[1]=cmd;
    arg[2]=param1;
    arg[3]=param2;
    DataGood=True
    wait=True
    t0=time.time()
    while(wait):
        if (GPIO.input(ppACK)!=0):  #Ensure that ACK is high before proceeding
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    GPIO.output(ppFRAME,True)       #Set the FRAME signal high
    time.sleep(0.000001)       #allow the uP some time to initialize the SPI
    spi.xfer(arg,500000,0)     #Send the fetch command
    t0=time.time()
    wait=True
    while(wait):
        if (GPIO.input(ppACK)==0):  #wait until the command is received and ACKnowledged
            wait=False
        if ((time.time()-t0)>0.05):   #timeout
            wait=False
            DataGood=False
    if DataGood:
        time.sleep(0.000001)     #allow the uP some time to initialize the SPI
        if(N>0):
            for i in range(N):          # bug in RPi5 SPIDEV limits block transfers to 64 bytes            
                blockBYTES[i*64:i*64+63]=spi.xfer3([0x55]*64,4000000,0)
            if (REM != 0):
                blockBYTES[N*64:N*64+(REM-1)]=spi.xfer3([0x55]*REM,4000000,0)
        else:
            if (REM != 0):
                blockBYTES[0:(REM-1)]=spi.xfer3([0x55]*REM,4000000,0)
    GPIO.output(ppFRAME,False)
    time.sleep(.00001)          #added for the RPi5 which is damn fast. DAQC2plate would sometimes miss FRAME going from high to low    
    return list(blockBYTES)


import spidev
import time
import site
import sys
import threading
from six.moves import input as raw_input
import subprocess
command = ["cat", "/proc/cpuinfo"]
output = subprocess.check_output(command)
for line in output.decode().splitlines():
    if "Model" in line:
        model = line.split(":")[1].strip()
        break    
#print(model)
if model.find("Raspberry Pi 5") != -1:
    import CMD5 as CMD 
else:
    import CMD0 as CMD


#Initialize
if (sys.version_info < (3,0,0)):
    sys.stderr.write("This library requires Python3")
    exit(1)
    
DIGIbaseADDR=0x58
    
localPath=site.getsitepackages()[0]
helpPath=localPath+'/piplates/DIGIhelp.txt'
#helpPath='DIGIhelp.txt'       #for development only
DIGIversion=2.0
# Version 1.0   -   initial release
# Version 2.0   -   Modified to support RPi5

RMAX = 2000
MAXADDR=8
digisPresent = list(range(8))
DataGood=False
lock = threading.Lock()
lock.acquire()

#==============================================================================#
# HELP Functions                                                               #
#==============================================================================#
def Help():
    help()

def HELP():
    help()  
    
def help():
    valid=True
    try:    
        f=open(helpPath,'r')
        while(valid):
            Count=0
            while (Count<20):
                s=f.readline()
                if (len(s)!=0):
                    print (s[:len(s)-1])
                    Count = Count + 1
                    if (Count==20):
                        Input=raw_input('press \"Enter\" for more...')                        
                else:
                    Count=100
                    valid=False
        f.close()
    except IOError:
        print ("Can't find help file.")

def getSWrev():
    return DIGIversion

#===============================================================================#   
# Digital Input Functions                                                       #
#===============================================================================#
def getDINbit(addr,bit):
    VerifyADDR(addr)
    VerifyDINchannel(bit-1)
    resp=ppCMD(addr,0x20,bit-1,0,1)
    if resp[0] > 0:
        return 1
    else:
        return 0
        
def getDINall(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x25,0,0,1)
    return resp[0]

#==============================================================================#
# Event Functions                                                              #
#==============================================================================#
def enableDINevent(addr, bit, edge):  # enable DIN interrupt
    VerifyADDR(addr)
    VerifyDINchannel(bit-1)
    bit = bit-1
    if ((edge=='f') or (edge=='F')):
        resp=ppCMD(addr,0x21,bit,0,0)
    if ((edge=='r') or (edge=='R')):
        resp=ppCMD(addr,0x22,bit,0,0)
    if ((edge=='b') or (edge=='B')):
        resp=ppCMD(addr,0x23,bit,0,0)

def disableDINevent(addr,bit):    # disable DIN interrupt
    VerifyADDR(addr)
    VerifyDINchannel(bit-1)
    resp=ppCMD(addr,0x24,bit-1,0,0)    

def eventEnable(addr):    #DIGIplate will pull down on INT pin if an enabled event occurs
    VerifyADDR(addr)
    resp=ppCMD(addr,0x04,0,0,0)

def eventDisable(addr):   #DIGIplate will not assert interrupts
    VerifyADDR(addr)
    resp=ppCMD(addr,0x05,0,0,0)
    
def getEVENTS(addr):  #read INT flag register in DIGIplate - this clears interrupt line and the register
    VerifyADDR(addr)
    resp=ppCMD(addr,0x06,0,0,2)
    value=((resp[0]<<8) + resp[1])
    return value

def check4EVENTS():
    stat=CMD.getSRQ()
    return stat
    
#==============================================================================#    
# FREQ Functions                                                               #
#==============================================================================#      
def getFREQ(addr,chan):
    global DataGood
    VerifyADDR(addr)
    assert ((chan>=1) and (chan<=6)),"Frequency input channel value out of range. Must be in the range of 1 to 6"
    freq=0
    resp=ppCMD(addr,0xC0,0,chan-1,2) #get the upper 16 bits
    #print (1, DataGood, (resp[0]<<24)+(resp[1]<<16), resp[2])
    if(DataGood):
        counts=(resp[0]<<24)+(resp[1]<<16)
        resp=ppCMD(addr,0xC0,1,chan-1,2) #get the lower 16 bits
        #print (2, DataGood, (resp[0]<<8)+resp[1], resp[2])
        if (DataGood):
            counts=counts+(resp[0]<<8)+resp[1]
            if (counts>0):
                freq=1000000.0/counts
            else:
                freq=0
    return round(freq,3)

def getFREQall(addr):
    global DataGood
    VerifyADDR(addr)
    freqList=6*[0]
    for i in range(6):
        chan=i+1
        freq=0
        resp=ppCMD(addr,0xC0,0,chan-1,2) #get the upper 16 bits
        if(DataGood):
            counts=(resp[0]<<24)+(resp[1]<<16)
            resp=ppCMD(addr,0xC0,1,chan-1,2) #get the lower 16 bits
            #print (2, DataGood, (resp[0]<<8)+resp[1], resp[2])
            if (DataGood):
                counts=counts+(resp[0]<<8)+resp[1]
                if (counts>0):
                    freq=1000000.0/counts
                else:
                    freq=0
                freqList[i]=round(freq,3)
    return freqList    
    
    
#==============================================================================#    
# LED Functions                                                                #
#==============================================================================#   
def setLED(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x60,0,0,0)

def clrLED(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x61,0,0,0)

def toggleLED(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x62,0,0,0)
  
#==============================================================================#    
# SYSTEM Functions                                                             #
#==============================================================================#     
def getID(addr):
    global DIGIbaseADDR
    addr=addr+DIGIbaseADDR
    return CMD.getID2(addr)


def getHWrev(addr):
    global DIGIbaseADDR
    VerifyADDR(addr)
    resp=ppCMD(addr,0x02,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0  
    
def getFWrev(addr):
    global DIGIbaseADDR
    VerifyADDR(addr)
    resp=ppCMD(addr,0x03,0,0,1)
    rev = resp[0]
    whole=float(rev>>4)
    point = float(rev&0x0F)
    return whole+point/10.0

def getVersion():
    return DIGIversion
        
#==============================================================================#    
# LOW Level Functions                                                          #
#==============================================================================#          
def VerifyDINchannel(din):
    assert ((din>=0) and (din<=7)),"Digital input channel value out of range. Must be in the range of 1 to 8"
    
def VerifyADDR(addr):
    assert ((addr>=0) and (addr<MAXADDR)),"DIGIplate address out of range"
    addr_str=str(addr)
    assert (digisPresent[addr]==1),"No DIGIplate found at address "+addr_str


def ppCMD(addr,cmd,param1,param2,bytes2return):
    global DIGIbaseADDR
    return CMD.ppCMD2(addr+DIGIbaseADDR,cmd,param1,param2,bytes2return)

def setINT(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0xF4,0,0,0)

def clrINT(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0xF5,0,0,0)
    
def getADDR(addr):
    global DIGIbaseADDR
    #resp = []
    resp=ppCMD(addr,0x00,0,0,1)
    #print(resp)
    if (CMD.DataGood):
        return resp[0]-DIGIbaseADDR
    else:
        return 8
    
def quietPoll():   
    global digisPresent
    ppFoundCount=0
    for i in range (0,8):
        digisPresent[i]=0
        rtn = getADDR(i)
        if (rtn==i):           
            digisPresent[i]=1
            ppFoundCount += 1

def RESET(addr):
    VerifyADDR(addr)
    resp=ppCMD(addr,0x0F,0,0,0)    
    time.sleep(.10)

quietPoll()    



import os
import sys
import subprocess

file_dir = os.path.dirname(__file__)
sys.path.append(file_dir)

command = ["cat", "/proc/cpuinfo"]
output = subprocess.check_output(command)
for line in output.decode().splitlines():
    if "Model" in line:
        model = line.split(":")[1].strip()
        break    
#print(model)
if model.find("Raspberry Pi 5") != -1:
    from BASE5 import * 
else:
    from BASE0 import *

#==============================================================================#	
# A2D Functions	     	                                              		   #
#==============================================================================#	
def getADC(addr,channel):
    VerifyADDR(addr)
    VerifyAINchannel(channel)
    resp=ppCMD(addr,0x30,channel-1,0,2)
    value=(256*resp[0]+resp[1])
    value=(value*5.10*2.4/4095)
    return round(value,3)

def getADCall(addr):
    value=list(range(4))
    VerifyADDR(addr)
    resp=ppCMD(addr,0x31,0,0,8)
    for i in range (0,4):
        value[i]=(256*resp[2*i]+resp[2*i+1])
        value[i]=round((value[i]*5.10*2.4/4095),3)
    return value    
 
def VerifyAINchannel(ain):
    assert ((ain>=1) and (ain<=4)),"Analog input channel value out of range. Must be in the range of 1 to 4"